<?php
//Lists all subfolder of a given folder into an array
function listSubFolders($dir, &$list) {
	global $assetDir;
    logMessage('Listing subfolders from: "' . $dir . '"');
    $dirs = glob($dir . DIRECTORY_SEPARATOR . '*', GLOB_ONLYDIR);

    foreach ($dirs as $dir) {
        if (!strpos($dir, DIRECTORY_SEPARATOR . $assetDir)) {
			if(is_writable($dir)){
				$list[] = $dir;
			} else {
				logMessage('Folder not writeable: ' . $dir . '"');
			}
			listSubFolders($dir, $list);
        }
    }
}

//Lists jpg, png, gif images of a given folder. If $recursive is set to 1 it also lists from  subfolders.
function listImages($recursive, $dir) {
    $imageList = array();
    if ($recursive == 0) {
        logMessage('Listing images from: "' . $dir . '"');
        $files = glob($dir . DIRECTORY_SEPARATOR . '*.{jpeg,JPEG,jpg,JPG,png,PNG,gif,GIF}', GLOB_BRACE);
        foreach ($files as $file) {
            $imageList[] = $file;
        }
    } elseif ($recursive == 1) {
        if (is_writable($dir)) {
            $folders = array($dir);
        } else {
            $folders = array();
            logMessage('Folder not writeable: ' . $dir . '"');
        }
        listSubFolders($dir, $folders);
        logMessage('Listing images from: "' . $dir . '", and it\'s subfolders.');
        foreach ($folders as $folder) {
            $files = glob($folder . DIRECTORY_SEPARATOR . '*.{jpeg,JPEG,jpg,JPG,png,PNG,gif,GIF}', GLOB_BRACE);
            foreach ($files as $file) {
                $imageList[] = $file;
            }
        }
    }
    return $imageList;
}
//Limits sizes to a given maximum amount.
function limitMaxPx($sizes, $maxPx){
	if(is_array($sizes)){
		$result = array();
		$c = count($sizes);
		for($i=0; $i < $c;$i++){
			if($sizes[$i] > $maxPx){
				$result[$i] = $maxPx;
			} else {
				$result[$i] = $sizes[$i];
			}
		}
	} else {
		if($sizes > $maxPx){
			$result = $maxPx;
		} else {
			$result = $sizes;
		}
	}
		return $result;
}
//Returns the new width & height based on user input.
function newSize($imageData, $resizeData, $maxPx) {
    list($width, $height) = $imageData;
    $method = $resizeData['method'];

	$sizes = limitMaxPx(array($resizeData['newwidth'], $resizeData['newheight'], $resizeData['lrgside']), $maxPx);
	list($newWidth, $newHeight, $lrgSide) = $sizes;

    if ($method == 0) {
        if ($newWidth == 0) {
            $newWidth = $width;
        }

        if ($newHeight == 0) {
            $newHeight = $height;
        }
    } elseif ($method == 1) {

        if ($width > $height) {
            $order = array("width", "height");
        } else {
            $order = array("height", "width");
        }

        $values = array('width' => $width, 'height' => $height);
        $newValues = $values;

        if ($values[$order[0]] != $lrgSide) {
            $newValues[$order[0]] = $lrgSide;
            $newValues[$order[1]] = round(($lrgSide / $values[$order[0]]) * $values[$order[1]]);
			if($newValues[$order[1]] <= 0) $newValues[$order[1]] = 1;
        }

        $newWidth = $newValues['width'];
        $newHeight = $newValues['height'];
    }
    return array($newWidth, $newHeight);
}
//Returns bytes from human readable form.
function minByteSize($minSize, $unit) {
    switch ($unit) {

        case 'GB':
            $minByteSize = pow(1024, 3) * $minSize;
            break;

        case 'MB':
            $minByteSize = pow(1024, 2) * $minSize;
            break;

        case 'kB':
            $minByteSize = 1024 * $minSize;
            break;

        default:
            $minByteSize = $minSize;
            break;
    }
    return $minByteSize;
}
//Formats bytes to a human readable form.
function formatBytes($bytes, $precision = 2) {
    $units = array('B', 'KB', 'MB', 'GB', 'TB');

    $bytes = max($bytes, 0);
    $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
    $pow = min($pow, count($units) - 1);

    $bytes /= pow(1024, $pow);

    return round($bytes, $precision) . ' ' . $units[$pow];
}
//Checks if resizing is necessary.
function check($imageData, $newSize, $filesize, $resizeData, $maxPx) {
    list($width, $height) = $imageData;
    list($newWidth, $newHeight) = $newSize;
    $minByteSize = $resizeData['minbyte'];
    $shrinkOnly = $resizeData['shrinkonly'];
    $type = $imageData[2];

	for($i = 0; $i<2; $i++){
		if($imageData[$i] > $maxPx){
			logMessage('Image too big for processing!');
			return false;
		}
	}

    if (!in_array(strtolower(image_type_to_mime_type($type)), array('image/jpeg', 'image/png', 'image/gif'))) {
        return false;
    }

    if ($minByteSize != 0 && $minByteSize > $filesize) {
        return False;
    }
    if ($width == $newWidth && $height == $newHeight) {
        return False;
    }
    if ($shrinkOnly == 1 and $newWidth > $width || $newHeight > $height) {
        return false;
    }
    return True;
}
//Resizes image.
function resizeImage($imageData, $newSize, $filename, $keepOld) {
    list($newWidth, $newHeight) = $newSize;
    list($width, $height, $type) = $imageData;

    logMessage('Resizing image: "' . $filename . '" from: "' . $width . ' x ' . $height . '" to: "' . $newWidth . ' x ' . $newHeight . '"');
    $ok = false;

    switch (strtolower(image_type_to_mime_type($type))) {

        case 'image/jpeg':
            $base = imagecreatetruecolor($newWidth, $newHeight);
            $source = imagecreatefromjpeg($filename);
            if ($keepOld == 1) {
                logMessage('Saving original images as: "' . $filename . '.old"');
                rename($filename, $filename . '.old');
            }
            imagecopyresampled($base, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
            if (imagejpeg($base, $filename)) {
                logMessage('Resizing successful');
                $ok = true;
            } else {
                logMessage('Error!: Something went wrong during resizing!');
            }
            imagedestroy($base);
            break;

        case 'image/png':
            $base = imagecreatetruecolor($newWidth, $newHeight);
            $source = imagecreatefrompng($filename);
            if ($keepOld == 1) {
                logMessage('Saving original images as: "' . $filename . '.old"');
                rename($filename, $filename . '.old');
            }

            imagealphablending($base, false);
            imagesavealpha($base, true);
            $transparent = imagecolorallocatealpha($base, 255, 255, 255, 127);
            imagefilledrectangle($base, 0, 0, $newWidth, $newHeight, $transparent);

            imagecopyresampled($base, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
            if (imagepng($base, $filename)) {
                logMessage('Resizing successful');
                $ok = true;
            } else {
                logMessage('Error!: Something went wrong during resizing!');
            }
            imagedestroy($base);
            break;

        case 'image/gif':
            $base = imagecreatetruecolor($newWidth, $newHeight);
            $source = imagecreatefromgif($filename);

			if ($keepOld == 1) {
                logMessage('Saving original images as: "' . $filename . '.old"');
                rename($filename, $filename . '.old');
            }

            imagecopyresampled($base, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
            if (imagegif($base, $filename)) {
                logMessage('Resizing successful');
                $ok = true;
            } else {
                logMessage('Error!: Something went wrong during resizing!');
            }
            imagedestroy($base);
            break;

        default:
            logMessage('Error!: Bad extension: "' . $filename . '"');
    }
    if ($ok == true) {
        return true;
    } else {
        return false;
    }
}

//Logging
function logMessage($message) {
	global $logName;
    $date = date('c');
    $message = $date . ' - ' . $message . "\r\n";
    file_put_contents($logName, $message, FILE_APPEND);
}
?>